PROC S_M()

; S_M - Search for and Modify text in current script in PAL editor.

; NOTE: Make sure the called modification script leaves the curson at least
;       one column to the right of the beginning of the last search string.

PRIVATE
   WorkMessage,
   InsFlag,
   UInput,
   SValue,
   MValue,
   SWidth,
   ConfirmFlag,
   NoneFlag,
   NoneMessage,
   FoundFlag,
   HoldRow,
   HoldColumn,
   WhereHold,
   HowMany,
   Bingo,
   LineHold,
   StringHold,
   LeftHold,
   RightHold,
   OldHold,
   TempLength,
   TempMark,
   PlainMark,
   I,
   J

; Assign variables.

InsFlag = FALSE
WorkMessage = "Searching & Modifying..."
NoneMessage = "No match found."
ConfirmFlag = FALSE
NoneFlag = TRUE
FoundFlag = FALSE
WhereHold = 22

; See if we're in the right mode.

IF SYSMODE() <> "Script" THEN
   BEEP
   RETURN
ENDIF

; Get string to search for.

CLEAR
MESSAGE
"Enter string, not case sensitive, for SEARCH/MODIFICATION (Esc to cancel)... "

@12,5
ACCEPT "A70"
   REQUIRED
   TO UInput

; Check for "Esc" entry.

IF NOT retval THEN
   CLEAR
   RETURN
ENDIF

SValue = UInput
SWidth = LEN(SValue)

; Get name of script to perform modifications.

CLEAR

@12,5
?? "Enter name of script to play when search value is located:  "

MESSAGE "(Esc to cancel)..."

@14,5
ACCEPT "A70"
   REQUIRED
   TO UInput

; Check for "Esc" entry.

IF NOT retval THEN
   CLEAR
   RETURN
ENDIF

MValue = UInput

; Ask if confirmation is desired.

WHILE (TRUE)

MESSAGE "Confirm each modification? (Y or N, Esc to cancel) "

@22,78
UInput = GETCHAR()

; Check for "Esc" entry.

IF UInput = 27 THEN
   RETURN
ENDIF

IF UInput = 89  OR
   UInput = 121 THEN

   ConfirmFlag = TRUE
   WorkMessage = "Searching..."
   QUITLOOP
ENDIF

IF UInput = 78  OR
   UInput = 110 THEN

   QUITLOOP
ENDIF

BEEP

ENDWHILE

; Store return variables, count number of script rows.

HoldRow = ROWNO()
HoldColumn = COLNO()
END
HowMany = ROWNO()

; Move to the top and begin working.

CLEAR
HOME

IF NOT ISINSERTMODE() THEN
   INS
   InsFlag = TRUE
ENDIF

@12,0
?? FORMAT("w80,ac",WorkMessage)

FOR I FROM 1 TO HowMany
   WHILE CHARWAITING()
      UInput = GETCHAR()

      IF UInput = 27 THEN
         QUITLOOP
      ENDIF
   ENDWHILE

   CTRLLEFT
   LeftHold = ""
   LineHold = CURSORLINE()
   StringHold = LineHold

   IF LineHold = "" THEN
      DOWN
      LOOP
   ENDIF

   WHILE (TRUE)
      Bingo = SEARCH(SValue,StringHold)

      IF Bingo > 0 THEN
         FoundFlag = TRUE
         LeftHold = LeftHold + SUBSTR(StringHold,1,Bingo-1)
         OldHold = SUBSTR(StringHold,Bingo,SWidth)
         TempLength = LEN(StringHold) - (Bingo + SWidth)
         TempMark = LEN(StringHold) - TempLength
         StringHold = SUBSTR(StringHold,TempMark,TempLength + 1)

         IF ConfirmFlag
            THEN
               PlainMark = LEN(LeftHold)
               CLEAR
               @WhereHold - 6,0
               ?? LeftHold
               STYLE BLINK
               @WhereHold - 6,PlainMark
               ?? OldHold
               STYLE
               PlainMark = PlainMark + SWidth
               @WhereHold - 6,PlainMark
               ?? StringHold
               STYLE REVERSE
               @WhereHold,52 - LEN(STRVAL(I))
               ?? "  Line " + STRVAL(I) + " - modify? (Y or N)  "
               @WhereHold,78

               UInput = GETCHAR()

               STYLE
               CLEAR

               IF UInput = 27 THEN
                  CLEAR

                  IF InsFlag THEN
                     INS
                  ENDIF

                  RETURN
               ENDIF

               @12,0
               ?? FORMAT("w80,ac",WorkMessage)

               IF (UInput = 89) OR (UInput = 121)
                  THEN
                     CTRLLEFT

                     IF LEN(LeftHold) > 0 THEN
                        FOR J FROM 1 TO LEN(LeftHold)
                           RIGHT
                        ENDFOR
                     ENDIF

                     EXECUTE "PLAY \"" + MValue + "\""

                     TempMark = COLNO()
                     LineHold = CURSORLINE()

                     IF (TempMark >= LEN(LineHold)) THEN
                        QUITLOOP
                     ENDIF

                     CTRLLEFT
                     LeftHold = SUBSTR(LineHold,1,TempMark)
                     TempLength = LEN(LineHold) - LEN(LeftHold)
                     StringHold = SUBSTR(LineHold,LEN(LeftHold) + 1,TempLength)
                  ELSE
                     LeftHold = LeftHold + OldHold
               ENDIF
            ELSE
               CTRLLEFT

               IF LEN(LeftHold) > 0 THEN
                  FOR J FROM 1 TO LEN(LeftHold)
                     RIGHT
                  ENDFOR
               ENDIF

               EXECUTE "PLAY \"" + MValue + "\""

               TempMark = COLNO()
               LineHold = CURSORLINE()

               IF (TempMark >= LEN(LineHold)) THEN
                  QUITLOOP
               ENDIF

               CTRLLEFT
               LeftHold = SUBSTR(LineHold,1,TempMark)
               TempLength = LEN(LineHold) - LEN(LeftHold)
               StringHold = SUBSTR(LineHold,LEN(LeftHold) + 1,TempLength)
         ENDIF

         LOOP
      ENDIF

      QUITLOOP
   ENDWHILE

   IF FoundFlag THEN
      NoneFlag = FALSE
   ENDIF

   CTRLLEFT
   DOWN
ENDFOR

; Return to former place in script.

HOME

FOR I FROM 1 TO HoldRow - 1
   DOWN
ENDFOR

CTRLLEFT

FOR I FROM 1 TO HoldColumn - 1
   RIGHT
ENDFOR

IF InsFlag THEN
   INS
ENDIF

WHILE CHARWAITING()
   UInput = GETCHAR()
ENDWHILE

; If none found, leave message.

IF NoneFlag THEN
   CLEAR
   @10,31
   ?? NoneMessage
   BEEP SLEEP 10 BEEP
   MESSAGE "Press a key to continue... "
   @22,78
   UInput = GETCHAR()
   CLEAR
ENDIF

ENDPROC ; S_M()

S_M()
