PROC S_R()

; S_R - Search and Replace a string in current script in PAL editor.

PRIVATE
   WorkMessage,
   InsFlag,
   UInput,
   SValue,
   RValue,
   SWidth,
   ConfirmFlag,
   NoneFlag,
   NoneMessage,
   FoundFlag,
   HoldRow,
   HoldColumn,
   WhereHold,
   HowMany,
   Bingo,
   LineHold,
   StringHold,
   LeftHold,
   RightHold,
   OldHold,
   TempLength,
   TempMark,
   PlainMark,
   I,
   J

; Assign variables.

InsFlag = FALSE
WorkMessage = "Searching & Replacing..."
NoneMessage = "No match found."
ConfirmFlag = FALSE
NoneFlag = TRUE
FoundFlag = FALSE
WhereHold = 22

; See if we're in the right mode.

IF SYSMODE() <> "Script" THEN
   BEEP
   RETURN
ENDIF

; Get string to search for.

CLEAR
MESSAGE "Enter string, not case sensitive, for SEARCH (Esc to cancel)... "

@12,5
ACCEPT "A70"
   REQUIRED
   TO UInput

; Check for "Esc" entry.

IF NOT retval THEN
   CLEAR
   RETURN
ENDIF

SValue = UInput
SWidth = LEN(SValue)

; Get replacement string.

CLEAR

@12,5
?? "Replace \"" + SValue + "\" with - "

MESSAGE
     "Enter exact REPLACEMENT string (Enter for search only, Esc to cancel)..."

@14,5
ACCEPT "A70"
   TO UInput

; Check for "Esc" entry.

IF NOT retval THEN
   CLEAR
   RETURN
ENDIF

RValue = UInput

; Check for null entry.

IF RValue = "" THEN
   WorkMessage = "Searching..."
   ConfirmFlag = TRUE
ENDIF

; Ask if confirmation is desired.

WHILE (TRUE)

IF RValue = "" THEN
   QUITLOOP
ENDIF

MESSAGE "Confirm each replacement? (Y or N, Esc to cancel) "

@22,78
UInput = GETCHAR()

; Check for "Esc" entry.

IF UInput = 27 THEN
   RETURN
ENDIF

IF UInput = 89  OR
   UInput = 121 THEN

   ConfirmFlag = TRUE
   WorkMessage = "Searching..."
   QUITLOOP
ENDIF

IF UInput = 78  OR
   UInput = 110 THEN

   QUITLOOP
ENDIF

BEEP

ENDWHILE

; Store return variables, count number of script rows.

HoldRow = ROWNO()
HoldColumn = COLNO()
END
HowMany = ROWNO()

; Move to the top and begin working.

CLEAR
HOME

IF NOT ISINSERTMODE() THEN
   INS
   InsFlag = TRUE
ENDIF

@12,0
?? FORMAT("w80,ac",WorkMessage)

FOR I FROM 1 TO HowMany
   WHILE CHARWAITING()
      UInput = GETCHAR()

      IF UInput = 27 THEN
         QUITLOOP
      ENDIF
   ENDWHILE

   CTRLLEFT
   LeftHold = ""
   LineHold = CURSORLINE()
   StringHold = LineHold

   IF LineHold = "" THEN
      DOWN
      LOOP
   ENDIF

   WHILE (TRUE)
      Bingo = SEARCH(SValue,StringHold)

      IF Bingo > 0 THEN
         FoundFlag = TRUE
         LeftHold = LeftHold + SUBSTR(StringHold,1,Bingo-1)
         OldHold = SUBSTR(StringHold,Bingo,SWidth)
         TempLength = LEN(StringHold) - (Bingo + SWidth)
         TempMark = LEN(StringHold) - TempLength
         StringHold = SUBSTR(StringHold,TempMark,TempLength + 1)

         IF ConfirmFlag
            THEN
               PlainMark = LEN(LeftHold)
               CLEAR
               @WhereHold - 6,0
               ?? LeftHold
               STYLE BLINK
               @WhereHold - 6,PlainMark
               ?? OldHold
               STYLE
               PlainMark = PlainMark + SWidth
               @WhereHold - 6,PlainMark
               ?? StringHold
               STYLE REVERSE

               IF RValue = ""
                  THEN
                     @WhereHold,47 - LEN(STRVAL(I))
                     ?? "  Line " + STRVAL(I) + " - halt search? (Y or N)  "
                  ELSE
                     @WhereHold,38 - LEN(STRVAL(I))
                     ?? "  Line " + STRVAL(I) +
                        " - replace? (Y or N, Esc to halt)  "
               ENDIF

               @WhereHold,78

               UInput = GETCHAR()

               STYLE
               CLEAR

               IF RValue = "" THEN
                  IF UInput = 89  OR
                     UInput = 121 THEN

                     IF InsFlag THEN
                        INS
                     ENDIF

                     CLEAR
                     RETURN
                  ENDIF
               ENDIF

               IF UInput = 27 THEN
                  IF InsFlag THEN
                     INS
                  ENDIF

                  CLEAR
                  RETURN
               ENDIF

               @12,0
               ?? FORMAT("w80,ac",WorkMessage)

               IF (UInput = 89) OR (UInput = 121)
                  THEN
                     IF RValue = ""
                        THEN
                           LeftHold = LeftHold + OldHold
                        ELSE
                           LeftHold = LeftHold + RValue
                     ENDIF
                  ELSE
                     LeftHold = LeftHold + OldHold
               ENDIF
            ELSE
               IF RValue = ""
                  THEN
                     LeftHold = LeftHold + OldHold
                  ELSE
                     LeftHold = LeftHold + RValue
               ENDIF
         ENDIF

         LOOP
      ENDIF

      QUITLOOP
   ENDWHILE

   IF FoundFlag THEN
      NoneFlag = FALSE
      FoundFlag = FALSE
      LineHold = LeftHold + StringHold
      CTRLLEFT

      IF ROWNO() = HowMany
         THEN
            DELETELINE
            CTRLEND
            ENTER
            TYPEIN LineHold
         ELSE
            DELETELINE
            ENTER
            UP
            TYPEIN LineHold
      ENDIF
   ENDIF

   DOWN
ENDFOR

; Return to former place in script.

HOME

FOR I FROM 1 TO HoldRow - 1
   DOWN
ENDFOR

CTRLLEFT

FOR I FROM 1 TO HoldColumn - 1
   RIGHT
ENDFOR

IF InsFlag THEN
   INS
ENDIF

WHILE CHARWAITING()
   UInput = GETCHAR()
ENDWHILE

; If none found, leave message.

IF NoneFlag THEN
   CLEAR
   @10,31
   ?? NoneMessage
   BEEP SLEEP 10 BEEP
   MESSAGE "Press a key to continue... "
   @22,78
   UInput = GETCHAR()
   CLEAR
ENDIF

ENDPROC ; S_R()

S_R()
